unit polje;

interface

uses SysUtils;

type barva = ( bPrazna, bNasprotnik, bJaz );

{ Podatkovni tip TPolje -- igralno polje za igro "4 v vrsto"

   uporaba:
    var
     p,q: TPolje;
     v,s: Integer;
     b: barva;

    p := TPolje.Create(13,17);        -- naredi polje dimenzij 13x17
    q := TPolje.Create(p);            -- naredi polje, katerega dimenzije in barve
                                          celic so enake polju p
    v := p.visina;                    -- prebere visino polja
    s := p.sirina;                    -- prebere sirino polja

    p.izpisi;                         -- izpise polje na zaslon
    p.dodaj(2, bNasprotnik);          -- najnizjo(!) prazno celico stolpca 2 pobarva
                                          z nasprotnikovo barvo
    p[1,2] := bJaz;                   -- celico (1,2) pobarva z mojo barvo
    b := p[3,4];                      -- prebere barvo celice (3,4)
}

type TPolje = class
private
  polje: array of array of barva;

  function getVisina(): Integer;
  function getSirina(): Integer;
  function getVrednost(x, y:Integer): barva;
  procedure setVrednost(x, y:Integer; val: barva);

public
  constructor Create(sirina, visina: Integer); overload;
  constructor Create(r: TPolje); overload;

  property sirina: Integer read getSirina;
  property visina: Integer read getVisina;
  property p[x,y: Integer]: barva read getVrednost write SetVrednost; default;

  procedure dodaj(stolpec: Integer; b: barva);
  procedure izpisi();
end;

implementation
  constructor TPolje.Create(sirina, visina: Integer);
  var
    i, j: Integer;
  begin
    SetLength(polje, sirina, visina);
    for i := 0 to sirina - 1 do
      for j := 0 to visina - 1 do
        polje[i,j] := bPrazna;
  end;

  constructor TPolje.Create(r: TPolje);
  var
    i, j: Integer;
  begin
    SetLength(polje, r.sirina, r.visina);
    for i := 0 to sirina - 1 do
      for j := 0 to visina - 1 do
        polje[i,j] := r[i,j];
  end;

  function TPolje.getVisina;
  begin
    getVisina := Length(polje[0]);
  end;

  function TPolje.getSirina;
  begin
   getSirina := Length(polje);
  end;

  function TPolje.getVrednost(x, y: Integer): barva;
  begin
    getVrednost := polje[x,y];
  end;

  procedure TPolje.setVrednost(x, y: Integer; val: barva);
  begin
    polje[x,y] := val;
  end;

  procedure TPolje.dodaj(stolpec: Integer; b: barva);
  var
    i: Integer;
  begin
    if stolpec = -1 then exit;
    for i := 0 to visina - 1 do
      if polje[stolpec,i] = bPrazna then begin
        polje[stolpec,i] := b;
         exit;
      end;
    raise Exception.Create('Stolpec je ze poln.');
  end;

  procedure TPolje.izpisi;
  var
    x,y: Integer;
  begin
    for y := visina - 1 downto 0 do begin
      for x := 0 to sirina - 1 do begin
        case polje[x,y] of
          bPrazna: Write(' ');
          bJaz: Write('*');
          bNasprotnik: Write('o');
        end;
      end;
      WriteLn;
    end;
  end;
end.
